import java.awt.Dimension;
import java.awt.Color;
import java.awt.Graphics;

import potential.*;

public class sim {
	
	// variables

	int steps;
	double[] vx;
	WaveVector W;
	double[] Amp;
	double Vmax;

	int [] re;
	int [] im;

	// arguments

	boolean wrap;
	double xrange;
	double dt;
	int npts;

	// derived constants

	double Vparticle;
	double dx;
	double xmin;
	double xmax;
	double Alpha;
	double Beta;
	double scalef;

	public sim(int sim_npts, double sim_dt, boolean sim_wrap,
			double sim_xrange, Potential sim_vx,
			int[] sim_re, int[] sim_im) {
		wrap = sim_wrap;
		npts = sim_npts;
		xrange = sim_xrange;
		dt = sim_dt;

		re = new int[npts];
		im = new int[npts];
		vx = new double[npts];

		System.arraycopy(sim_re, 0, re, 0, npts);
		System.arraycopy(sim_im, 0, im, 0, npts);
		sim_vx.render(vx);

		dx = xrange / (npts-1);
		steps = 0;
		Alpha = (Physics.hbar * dt) / (2 * Physics.me * dx * dx);
		Beta = (-2.0 * dt) / Physics.hbar;
		Vparticle = 2.0;
	}
	
	void update(int sign, int[] Wv1, int[] Wv2) {
		for(int i = 0; i<npts; i++) {
			int l = (i==0) ? (wrap ? Wv2[npts-1] : 0) : Wv2[i-1];
			int r = (i==npts-1) ? (wrap ? Wv2[0] : 0) : Wv2[i+1];
			int delta = (int)(
				Alpha * (l - (Wv2[i]<<1) + r) +
				Beta * (Wv2[i] * vx[i]));
			Wv1[i] += (sign > 0) ? (int)delta : -(int)delta;
		}
	}

	public void step(boolean direction, int ndt) {
		int[] a = direction ? re : im;
		int[] b = direction ? im : re;
		for (int i=0; i<ndt; i++) {
			update(-1, a, b);
			update(+1, b, a);
		}
		steps += direction ? ndt : -ndt;
	}

	public void render(Graphics g, Dimension d) {
		g.setColor(Color.black);
		g.fillRect(0, 0, d.width, d.height);
		g.setColor(Color.blue);
		paintWaveVector(g, d, re);
		g.setColor(Color.yellow);
		paintWaveVector(g, d, im);
		g.setColor(Color.white);
		double scale = 3e18 / (Vparticle*2) * (d.height/2);
		for(int i=0; i<npts-1; i++) {
			g.drawLine(
				(i*d.width/npts), 
				(d.height/2) - (int)(vx[i]*scale),
				((i+1)*d.width/npts), 
				(d.height/2) - (int)(vx[i+1]*scale)
			);
		}
	}

	public void paintWaveVector(Graphics g, Dimension d, int [] v) {
		double scale = (double)(d.height) / (double)(Integer.MAX_VALUE);
		for (int i=0; i<npts-1; i++) {
			g.drawLine(
				(i*d.width/npts), 
				(d.height/2) - (int)(v[i] * scale),
				((i+1)*d.width/npts), 
				(d.height/2) - (int)(v[i+1] * scale)
			);
		}
	}
}